@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Drager
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,18 +20,12 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #ifndef EDITSERVERLISTDLG_H
 #define EDITSERVERLISTDLG_H
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "EditServerListDlg.h"
-#endif
-
-#include <wx/defs.h>		// Needed before any other wx/*.h
-#include <wx/dialog.h>		// Needed for wxDialog
 
 class wxTextCtrl;
 
@@ -53,7 +47,7 @@ protected:
 
 private:
     DECLARE_EVENT_TABLE()
-    DECLARE_DYNAMIC_CLASS(EditServerListDlg)
 };
 
 #endif // EDITSERVERLISTDLG_H
+// File_checked_for_headers
