@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,27 +20,18 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #ifndef KNOWNFILELIST_H
 #define KNOWNFILELIST_H
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "KnownFileList.h"
-#endif
 
-#include <wx/defs.h>		// Needed before any other wx/*.h
-#include <wx/hashmap.h>		// Needed for WX_DECLARE_HASH_MAP, wxStringHash and wxStringEqual
-#include <wx/thread.h>		// Needed for wxMutex
-
-#include <list>
-
-#include "Types.h"		// Needed for uint16 and uint32
 #include "SharedFileList.h" // CKnownFileMap
 
 
-class CKnownFile;	
+class CKnownFile;
+class CPath;
 
 class CKnownFileList
 {
@@ -51,12 +42,15 @@ public:
 	bool	Init();
 	void	Save();
 	void	Clear();
-	CKnownFile*	FindKnownFile(wxString filename,uint32 in_date,uint32 in_size);
-	CKnownFile*   FindKnownFileByID(const CMD4Hash& hash);
+	CKnownFile* FindKnownFile(
+		const CPath& filename,
+		time_t in_date,
+		uint64 in_size);
+	CKnownFile* FindKnownFileByID(const CMD4Hash& hash);
 	bool	IsKnownFile(const CKnownFile* file);
 
 	uint16 requested;
-	uint32 transfered;
+	uint32 transferred;
 	uint16 accepted;
 
 private:
@@ -64,11 +58,21 @@ private:
 
 	bool	Append(CKnownFile*);
 
-	CKnownFile* IsOnDuplicates(wxString filename,uint32 in_date,uint32 in_size) const;
+	CKnownFile *IsOnDuplicates(
+		const CPath& filename,
+		uint32 in_date,
+		uint64 in_size) const;
+
+	bool KnownFileMatches(
+		CKnownFile *knownFile,
+		const CPath& filename,
+		uint32 in_date,
+		uint64 in_size) const;
 
 	typedef std::list<CKnownFile*> KnownFileList;
-	KnownFileList	m_duplicates;
-	CKnownFileMap	m_map;
+	KnownFileList	m_duplicateFileList;
+	CKnownFileMap	m_knownFileMap;
 };
 
 #endif // KNOWNFILELIST_H
+// File_checked_for_headers
