@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2004-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2004-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -19,19 +19,18 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "MuleGifCtrl.h"
-#endif
 
 #include <wx/mstream.h>
 #include <wx/gifdecod.h>
-#include <wx/window.h>
 #include <wx/dcbuffer.h>
 
+
 #include "MuleGifCtrl.h"
+#include "Types.h"
+
 
 BEGIN_EVENT_TABLE(MuleGifCtrl, wxControl)
 	EVT_TIMER(GIFTIMERID, MuleGifCtrl::OnTimer)
@@ -39,13 +38,40 @@ BEGIN_EVENT_TABLE(MuleGifCtrl, wxControl)
 	EVT_ERASE_BACKGROUND(MuleGifCtrl::OnErase)
 END_EVENT_TABLE()
 
+class MuleGIFDecoder : public wxGIFDecoder
+{
+public:
+	MuleGIFDecoder()
+	{
+		m_nframe = 0;
+	}
+	
+	~MuleGIFDecoder() { }
+	
+	void GoFirstFrame() { m_nframe = 0; }
+	void GoNextFrame() { m_nframe < GetFrameCount() - 1 ? m_nframe++ : m_nframe = 0; }
+	void GoLastFrame() { m_nframe = GetFrameCount() - 1; }
+	
+	void ConvertToImage(wxImage* image) { wxGIFDecoder::ConvertToImage(m_nframe, image); }
+	
+	long GetDelay() { return wxGIFDecoder::GetDelay(m_nframe); }
+		
+private:
+	uint32_t m_nframe;
+};
 
-MuleGifCtrl::MuleGifCtrl( wxWindow *parent, wxWindowID id, const wxPoint& pos,
-                          const wxSize& size, long style, const wxValidator& validator,
-                          const wxString& name )
-: wxControl( parent, id, pos, size, style, validator, name ),
-  m_decoder( NULL ),
-  m_timer( this, GIFTIMERID )
+MuleGifCtrl::MuleGifCtrl(
+	wxWindow *parent,
+	wxWindowID id,
+	const wxPoint& pos,
+	const wxSize& size,
+	long style,
+	const wxValidator& validator,
+	const wxString& name)
+:
+wxControl(parent, id, pos, size, style, validator, name),
+m_decoder(NULL),
+m_timer(this, GIFTIMERID)
 {
 }
 
@@ -53,8 +79,7 @@ MuleGifCtrl::MuleGifCtrl( wxWindow *parent, wxWindowID id, const wxPoint& pos,
 MuleGifCtrl::~MuleGifCtrl()
 {
 	m_timer.Stop();
-
-	if ( m_decoder ) {
+	if (m_decoder) {
 		delete m_decoder;
 		m_decoder = NULL;
 	}
@@ -68,32 +93,35 @@ bool MuleGifCtrl::LoadData(const char* data, int size)
    		delete m_decoder;
   		m_decoder = NULL;
 	}
-
+	
   	wxMemoryInputStream stream(data, size);
-  	m_decoder = new wxGIFDecoder(&stream, TRUE);
-  	if ( m_decoder->ReadGIF() != wxGIF_OK ) {
+  	m_decoder = new MuleGIFDecoder();
+  	if ( m_decoder->LoadGIF(stream) != wxGIF_OK ) {
    		delete m_decoder;
    		m_decoder = NULL;
    		return false;
   	}
-
+	
 	m_decoder->GoFirstFrame();
 	wxImage frame;
 	m_decoder->ConvertToImage( &frame );
 	m_frame = wxBitmap(frame);
-
+	
 	return true;
 }
 
 
 void MuleGifCtrl::Start()
 {
-	if ( m_decoder && m_decoder->IsAnimation() ) {
+	if (m_decoder && m_decoder->IsAnimation()) {
 		m_timer.Stop();
 		m_decoder->GoLastFrame();
-
+#if wxCHECK_VERSION(2, 9, 0)
+		wxTimerEvent evt(m_timer);
+#else
 		wxTimerEvent evt;
-		OnTimer( evt );
+#endif
+		OnTimer(evt);
 	}
 }
 
@@ -106,38 +134,43 @@ void MuleGifCtrl::Stop()
 
 wxSize MuleGifCtrl::GetBestSize()
 {
-	return wxSize( m_decoder->GetWidth(), m_decoder->GetHeight() );
+	return m_decoder->GetAnimationSize();
 }
 
 
-void MuleGifCtrl::OnTimer( wxTimerEvent& WXUNUSED(event) )
+void MuleGifCtrl::OnTimer(wxTimerEvent& WXUNUSED(event))
 {
-	if ( m_decoder ) {
-		if ( m_decoder->IsAnimation() )
-			m_decoder->GoNextFrame( true );
-
+	if (m_decoder) {
+		if (m_decoder->IsAnimation()) {
+			m_decoder->GoNextFrame();
+		}
+		
 		wxImage frame;
-		m_decoder->ConvertToImage( &frame );
+		m_decoder->ConvertToImage(&frame);
 		m_frame = wxBitmap(frame);
-
+		
 		Refresh();
-
-		if ( m_decoder->IsAnimation() )
-			m_timer.Start( m_decoder->GetDelay(), true );
+		
+		if (m_decoder->IsAnimation()) {
+			m_timer.Start(m_decoder->GetDelay(), true);
+		}
 	}
 }
 
 
-void MuleGifCtrl::OnPaint( wxPaintEvent& WXUNUSED(event) )
+void MuleGifCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
 	wxBufferedPaintDC dc(this);
-
-    wxSize size = GetClientSize();
-	int x = (size.GetWidth()-m_frame.GetWidth())/2;
-	int y = (size.GetHeight()-m_frame.GetHeight())/2;
-
-	dc.SetBackground( wxBrush( GetBackgroundColour(), wxSOLID ));
+	
+	wxSize clientsize = GetClientSize();
+	wxSize gifsize = m_decoder->GetAnimationSize();
+	int x = (clientsize.GetWidth()-gifsize.GetWidth())/2;
+	int y = (clientsize.GetHeight()-gifsize.GetHeight())/2;
+	
+	dc.SetBackground(wxBrush(GetBackgroundColour(), wxSOLID));
 	dc.Clear();
-	dc.DrawBitmap( m_frame, x, y, true);
+	dc.DrawBitmap(m_frame, x, y, true);
 }
 
+
+// File_checked_for_headers
