@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,28 +20,22 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #ifndef OSCOPECTRL_H
 #define OSCOPECTRL_H
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "OScopeCtrl.h"
-#endif
-
 #ifndef NULL
 #define NULL 0
 #endif
 
-#include <wx/defs.h>		// Needed before any other wx/*.h
 #include <wx/control.h>		// Needed for wxControl
 #include <wx/timer.h>		// Needed for wxTimer
 #include <wx/pen.h>
-#include <wx/brush.h>
+#include <wx/bitmap.h>
 
-#include "Types.h"		// Needed for RECT
-#include "Color.h"			// Needed for COLORREF and RGB
+#include "Color.h"		// Needed for COLORREF and RGB
 #include "Statistics.h"		// Needed for StatsGraphType
 
 class wxMemoryDC;
@@ -56,28 +50,23 @@ class COScopeCtrl : public wxControl
 	friend class CStatisticsDlg;
 		
 public:
-	COScopeCtrl(int NTrends, int nDecimals, wxWindow* parent=NULL);
+	COScopeCtrl(int NTrends, int nDecimals, StatsGraphType type, wxWindow* parent = NULL);
 	~COScopeCtrl();
 
 	void SetRange(float dLower, float dUpper, unsigned iTrend = 0);
 	void SetRanges(float dLower, float dUpper);
-	void SetXUnits(const wxString& string, const wxString& XMin = wxT(""), const wxString& XMax = wxT(""));
-	void SetYUnits(const wxString& string, const wxString& YMin = wxT(""), const wxString& YMax = wxT(""));
+	void SetYUnits(const wxString& string,
+		const wxString& YMin = wxEmptyString, const wxString& YMax = wxEmptyString);
 	void SetBackgroundColor(COLORREF color);
 	void SetGridColor(COLORREF color);
 	void SetPlotColor(COLORREF color, unsigned iTrend = 0);
-	COLORREF GetPlotColor(unsigned iTrend = 0)  {return pdsTrends[iTrend].crPlot;}
-	float GetUpperLimit()	{ return pdsTrends[0].fUpperLimit; }
+	float GetUpperLimit()		{ return pdsTrends[0].fUpperLimit; }
 	void Reset(double sNewPeriod);
 	void Stop();
 	void RecreateGraph(bool bRefresh=true);
 	void RecreateGrid();
-	void AppendPoints(double sTimestamp, const float *apf[]);
-	void DelayPoints()	{ nDelayedPoints++; }
-	unsigned GetPlotHeightPixels()		{return nPlotHeight;}
-	unsigned GetPlotWidthPoints()		{return nPlotWidth/nShiftPixels;}
-	wxBitmap* GetBitmapPlot()			{return bmapPlot;};
-	wxBitmap* GetBitmapGrid()			{return bmapGrid;};
+	void AppendPoints(double sTimestamp, const std::vector<float *> &apf);
+	void DelayPoints()		{ nDelayedPoints++; }
 
 	StatsGraphType graph_type;
 	
@@ -88,10 +77,10 @@ public:
 	unsigned nShiftPixels;         // amount to shift with each new point 
 	unsigned nYDecimals;
 
-	wxString strXUnits, strXMin, strXMax;
+	wxString strXUnits;
 	wxString strYUnits, strYMin, strYMax;
-	COLORREF crBackground;
-	COLORREF crGrid;      
+	wxColour m_bgColour;
+	wxColour m_gridColour;
 
 	typedef struct PlotDataStruct {
 		COLORREF crPlot;	       // data plot color  
@@ -108,26 +97,16 @@ protected:
 	DECLARE_EVENT_TABLE()
 	PlotData_t *pdsTrends;
 
-	unsigned nClientHeight;
-	unsigned nClientWidth;
-	int nPlotHeight;
-	int nPlotWidth;
+	wxRect	m_rectClient;
+	wxRect	m_rectPlot;
+	wxBrush	brushBack;
+	wxBitmap m_bmapGrid;
+	wxBitmap m_bmapPlot;
 
-	RECT  rectClient;
-	RECT  rectPlot;
-	wxBrush brushBack;
-
-	wxMemoryDC* dcGrid;
-	wxMemoryDC* dcPlot;
-	wxBitmap* bmapOldGrid;
-	wxBitmap* bmapOldPlot;
-	wxBitmap* bmapGrid;
-	wxBitmap* bmapPlot;
 	void InvalidateGraph()	{ InvalidateCtrl(true, false); }
 	void InvalidateGrid()	{ InvalidateCtrl(false, true); }
 
 private:
-	int oldwidth, oldheight;
 	bool bRecreateGrid, bRecreateGraph, bStopped;
 	int nDelayedPoints;
 	double sLastTimestamp;
@@ -138,9 +117,10 @@ private:
 	void OnSize(wxSizeEvent& evt);
 	void ShiftGraph(unsigned cntPoints);
 	void PlotHistory(unsigned cntPoints, bool bShiftGraph, bool bRefresh);
-	void DrawPoints(const float *apf[], unsigned cntPoints);
+	void DrawPoints(const std::vector<float *> &apf, unsigned cntPoints);
 	unsigned GetPlotY(float fPlot, PlotData_t* ppds);
 	void InvalidateCtrl(bool bInvalidateGraph = true, bool bInvalidateGrid = true);
 };
 
 #endif // OSCOPECTRL_H
+// File_checked_for_headers
