@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,34 +20,32 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "ServerConnect.h"
-#endif
+#include "ServerConnect.h"	// Interface declarations.
+
+#include <protocol/Protocols.h>
+#include <protocol/ed2k/ClientSoftware.h> // Sometimes we reply with TCP packets.
+#include <tags/ClientTags.h>
+#include <common/ClientVersion.h>
+#include <common/EventIDs.h>
 
-#include "ServerConnect.h"		// Interface declarations.
 #include "SearchList.h"		// Needed for CSearchList
-#include "GetTickCount.h"	// Needed for GetTickCount
-#include "UploadQueue.h"	// Needed for CUploadQueue
-#include "ServerUDPSocket.h"		// Needed for CServerUDPSocket
+#include "ServerUDPSocket.h"	// Needed for CServerUDPSocket
 #include "SharedFileList.h"	// Needed for CSharedFileList
 #include "Packet.h"		// Needed for CTag
-#include "OPCodes.h"		// Needed for CT_NAME
-#include "SafeFile.h"		// Needed for CSafeMemFile
-#include "OtherFunctions.h"	// Needed for GetTickCount
+#include "MemFile.h"		// Needed for CMemFile
 #include "ServerSocket.h"	// Needed for CServerSocket
 #include "ListenSocket.h"	// Needed for CListenSocket
 #include "Server.h"		// Needed for CServer
 #include "amule.h"		// Needed for theApp
 #include "ServerList.h"		// Needed for CServerList
 #include "Preferences.h"	// Needed for CPreferences
-#include "updownclient.h"	// for SO_AMULE
-#include "Statistics.h"
-#include "NetworkFunctions.h" // for StringHosttoUint32
+#include "Statistics.h"		// Needed for theStats
 #include "Logger.h"
-#include "Format.h"
+#include "GuiEvents.h"		// Needed for Notify_*
+#include <common/Format.h>
 
 
 //#define DEBUG_CLIENT_PROTOCOL
@@ -55,79 +53,91 @@
 
 void CServerConnect::TryAnotherConnectionrequest()
 {
-	if ( connectionattemps.size() < (( thePrefs::IsSafeServerConnectEnabled()) ? 1 : 2) ) {
+	if ( connectionattemps.size() < (unsigned)(( thePrefs::IsSafeServerConnectEnabled()) ? 1 : 2) ) {
 	
-		CServer*  next_server = used_list->GetNextServer();
+		CServer*  next_server = used_list->GetNextServer(m_bTryObfuscated);
 
-		if (!next_server)
-		{
+		if ( thePrefs::AutoConnectStaticOnly() ) {
+			while (next_server && !next_server->IsStaticMember()) {
+				next_server = used_list->GetNextServer(m_bTryObfuscated);
+			}
+		}
+
+		if (!next_server) {
 			if ( connectionattemps.empty() ) {
-				AddLogLineM(true, _("Failed to connect to all servers listed. Making another pass."));
-				
-				ConnectToAnyServer( lastStartAt );
+				if (m_bTryObfuscated && !thePrefs::IsClientCryptLayerRequired()){
+					AddLogLineM(true, _("Failed to connect to all obfuscated servers listed. Making another pass without obfuscation."));					
+					// try all servers on the non-obfuscated port next
+					m_bTryObfuscated = false;
+					ConnectToAnyServer( false, true);
+				} else {					
+					AddLogLineM(true, _("Failed to connect to all servers listed. Making another pass."));
+					ConnectToAnyServer( false );
+				}
 			}
 			return;
 		}
 
-		// Barry - Only auto-connect to static server option
-		if ( thePrefs::AutoConnectStaticOnly() ) {
-			if ( next_server->IsStaticMember() )
-                ConnectToServer(next_server, true);
-		} else {
-			ConnectToServer(next_server, true);
-		}
+		ConnectToServer(next_server, true, !m_bTryObfuscated);
 	}
 }
 
-
-void CServerConnect::ConnectToAnyServer(uint32 startAt,bool prioSort,bool isAuto)
+void CServerConnect::ConnectToAnyServer(bool prioSort, bool bNoCrypt)
 {
-	lastStartAt=startAt;
+	if (!thePrefs::GetNetworkED2K()){
+		AddLogLineM(true,_("eD2k network disabled on preferences, not connecting."));
+		return;
+	}
+
 	StopConnectionTry();
 	Disconnect();
 	connecting = true;
 	singleconnecting = false;
-	Notify_ShowConnState(false,wxEmptyString);
-
-
+	m_bTryObfuscated = thePrefs::IsServerCryptLayerTCPRequested() && !bNoCrypt;
+		
 	// Barry - Only auto-connect to static server option
-	if (thePrefs::AutoConnectStaticOnly() && isAuto)
-	{
+	if (thePrefs::AutoConnectStaticOnly()) {
 		bool anystatic = false;
 		CServer *next_server; 
-		used_list->SetServerPosition( startAt );
-		while ((next_server = used_list->GetNextServer()) != NULL)
-		{
-			if (next_server->IsStaticMember())
-			{
+		used_list->ResetServerPos();
+		while ((next_server = used_list->GetNextServer(false)) != NULL) {
+			if (next_server->IsStaticMember()) {
 				anystatic = true;
 				break;
 			}
 		}
-		if (!anystatic)
-		{
+		if (!anystatic) {
 			connecting = false;
-			AddLogLineM(true,_("No valid servers to connect in serverlist found"));
+			AddLogLineM(true,_("No valid servers to which to connect found in server list"));
 			return;
 		}
 	}
 
-	used_list->SetServerPosition( startAt );
-	if ( thePrefs::Score() && prioSort ) used_list->Sort();
+	if ( thePrefs::Score() && prioSort ) {
+		used_list->Sort();
+	}
+	
+	used_list->ResetServerPos();
 
-	if (used_list->GetServerCount()==0 ){
+	if (used_list->GetServerCount()==0 ) {
 		connecting = false;
-		AddLogLineM(true,_("No valid servers to connect in serverlist found"));
+		AddLogLineM(true,_("No valid servers to which to connect found in server list"));
 		return;
 	}
-	theApp.listensocket->Process();
+	
+	theApp->listensocket->Process();
 
 	TryAnotherConnectionrequest();
 }
 
 
-void CServerConnect::ConnectToServer(CServer* server, bool multiconnect)
-{
+void CServerConnect::ConnectToServer(CServer* server, bool multiconnect, bool bNoCrypt)
+{	
+	if (!thePrefs::GetNetworkED2K()){
+		AddLogLineM(true,_("eD2k network disabled on preferences, not connecting."));
+		return;
+	}
+	
 	if (!multiconnect) {
 		StopConnectionTry();
 		Disconnect();
@@ -135,10 +145,9 @@ void CServerConnect::ConnectToServer(CServer* server, bool multiconnect)
 	connecting = true;
 	singleconnecting = !multiconnect;
 
-//#ifdef TESTING_PROXY
 	CServerSocket* newsocket = new CServerSocket(this, thePrefs::GetProxyData());
-	m_lstOpenSockets.AddTail(newsocket);
-	newsocket->ConnectToServer(server);
+	m_lstOpenSockets.push_back(newsocket);
+	newsocket->ConnectToServer(server, bNoCrypt);
 
 	connectionattemps[GetTickCount()] = newsocket;
 }
@@ -156,8 +165,8 @@ void CServerConnect::StopConnectionTry()
 	} 
 
 	// close all currenty opened sockets except the one which is connected to our current server
-	for( POSITION pos = m_lstOpenSockets.GetHeadPosition(); pos != NULL; ) {
-		CServerSocket* pSck = m_lstOpenSockets.GetNext(pos);
+	for(SocketsList::iterator it = m_lstOpenSockets.begin(); it != m_lstOpenSockets.end(); ) {
+		CServerSocket *pSck = *it++;
 		if (pSck == connectedsocket)		// don't destroy socket which is connected to server
 			continue;
 		if (pSck->m_bIsDeleting == false)	// don't destroy socket if it is going to destroy itself later on
@@ -165,13 +174,6 @@ void CServerConnect::StopConnectionTry()
 	}
 }
 
-
-#define CAPABLE_ZLIB 0x01
-#define CAPABLE_IP_IN_LOGIN_FRAME 0x02
-#define CAPABLE_AUXPORT 0x04
-#define CAPABLE_NEWTAGS 0x08
-#define CAPABLE_UNICODE 0x10
-
 void CServerConnect::ConnectionEstablished(CServerSocket* sender)
 {
 	if (connecting == false)
@@ -180,7 +182,6 @@ void CServerConnect::ConnectionEstablished(CServerSocket* sender)
 		DestroySocket(sender);
 		return;
 	}	
-	InitLocalIP();
 	
 	if (sender->GetConnectionState() == CS_WAITFORLOGIN) {
 		AddLogLineM(false, CFormat( _("Connected to %s (%s:%i)") )
@@ -189,49 +190,62 @@ void CServerConnect::ConnectionEstablished(CServerSocket* sender)
 			% sender->cur_server->GetPort() );
 
 		//send loginpacket
-		CServer* update = theApp.serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
+		CServer* update = theApp->serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
 		if (update){
 			update->ResetFailedCount();
 			Notify_ServerRefresh( update );
 		}
 		
-		CSafeMemFile data(256);
-		data.WriteHash16(thePrefs::GetUserHash());
+		CMemFile data(256);
+		data.WriteHash(thePrefs::GetUserHash());
 		// Why pass an ID, if we are loggin in?
 		data.WriteUInt32(GetClientID());
 		data.WriteUInt16(thePrefs::GetPort());
-		data.WriteUInt32(5); // tagcount
+		data.WriteUInt32(4); // tagcount
 
-		CTag tagname(CT_NAME,thePrefs::GetUserNick());
+		// Kry - Server doesn't support VBT tags afaik.
+		// Not to mention we don't know its flags yet
+		
+		CTagString tagname(CT_NAME,thePrefs::GetUserNick());
 		tagname.WriteTagToFile(&data);
 
-		CTag tagversion(CT_VERSION,EDONKEYVERSION);
+		CTagInt32 tagversion(CT_VERSION,EDONKEYVERSION);
 		tagversion.WriteTagToFile(&data);
+
+		uint32 dwCryptFlags = 0;
 		
-		CTag tagport(CT_PORT,thePrefs::GetPort());
-		tagport.WriteTagToFile(&data);
+		if (thePrefs::IsClientCryptLayerSupported()) {
+			dwCryptFlags |= SRVCAP_SUPPORTCRYPT;
+		}
+		
+		if (thePrefs::IsClientCryptLayerRequested()) {
+			dwCryptFlags |= SRVCAP_REQUESTCRYPT;
+		}
+		
+		if (thePrefs::IsClientCryptLayerRequired()) {
+			dwCryptFlags |= SRVCAP_REQUIRECRYPT;
+		}
 		
 		// FLAGS for server connection
-		CTag tagflags(CT_SERVER_FLAGS, CAPABLE_ZLIB 
-													| CAPABLE_AUXPORT 
-													| CAPABLE_NEWTAGS 
-#if wxUSE_UNICODE													
-													| CAPABLE_UNICODE
-#endif													
-													); 
+		CTagInt32 tagflags(CT_SERVER_FLAGS, CAPABLE_ZLIB 
+								| CAPABLE_AUXPORT 
+								| CAPABLE_NEWTAGS 
+								| CAPABLE_UNICODE
+								| CAPABLE_LARGEFILES
+								| dwCryptFlags
+											); 
 		
 		tagflags.WriteTagToFile(&data);
 
 		// eMule Version (14-Mar-2004: requested by lugdunummaster (need for LowID clients which have no chance 
 		// to send an Hello packet to the server during the callback test))
-		CTag tagMuleVersion(CT_EMULE_VERSION, 
+		CTagInt32 tagMuleVersion(CT_EMULE_VERSION, 
 			(SO_AMULE	<< 24) |
 			make_full_ed2k_version(VERSION_MJR, VERSION_MIN, VERSION_UPDATE)
 			 );
 		tagMuleVersion.WriteTagToFile(&data);
 
-		CPacket* packet = new CPacket(&data);
-		packet->SetOpCode(OP_LOGINREQUEST);
+		CPacket* packet = new CPacket(data, OP_EDONKEYPROT, OP_LOGINREQUEST);
 		#ifdef DEBUG_CLIENT_PROTOCOL
 		AddLogLineM(true,wxT("Client: OP_LOGINREQUEST"));
 		AddLogLineM(true,wxString(wxT("        Hash     : ")) << thePrefs::GetUserHash().Encode());
@@ -240,37 +254,38 @@ void CServerConnect::ConnectionEstablished(CServerSocket* sender)
 		AddLogLineM(true,wxString(wxT("        User Nick: ")) << thePrefs::GetUserNick());
 		AddLogLineM(true,wxString(wxT("        Edonkey  : ")) << EDONKEYVERSION);
 		#endif
-		theApp.statistics->AddUpDataOverheadServer(packet->GetPacketSize());
+		theStats::AddUpOverheadServer(packet->GetPacketSize());
 		SendPacket(packet, true, sender);
 	} else if (sender->GetConnectionState() == CS_CONNECTED){
-		theApp.statistics->AddReconnect();
-		theApp.statistics->SetServerConnectTime(GetTickCount64());
+		theStats::AddReconnect();
+		theStats::GetServerConnectTimer()->ResetTimer();
 		connected = true;
 		AddLogLineM(true, CFormat( _("Connection established on: %s") ) % sender->cur_server->GetListName());
 		connectedsocket = sender;
-		Notify_ShowConnState(true,connectedsocket->cur_server->GetListName());
 		
 		StopConnectionTry();
 		
-		CServer* update = theApp.serverlist->GetServerByAddress(connectedsocket->cur_server->GetAddress(),sender->cur_server->GetPort());
-		if ( update )
+		CServer* update = theApp->serverlist->GetServerByAddress(connectedsocket->cur_server->GetAddress(),sender->cur_server->GetPort());
+		if ( update ) {
 			Notify_ServerHighlight(update, true);
+		}
 		
-		theApp.sharedfiles->ClearED2KPublishInfo();
+		theApp->sharedfiles->ClearED2KPublishInfo();
 
 		Notify_ServerRemoveDead();
 		
 		// tecxx 1609 2002 - serverlist update
-		if (thePrefs::AddServersFromServer())
-		{
-			CPacket* packet = new CPacket(OP_GETSERVERLIST,0);
-			theApp.statistics->AddUpDataOverheadServer(packet->GetPacketSize());
+		if (thePrefs::AddServersFromServer()) {
+			CPacket* packet = new CPacket(OP_GETSERVERLIST, 0, OP_EDONKEYPROT);
+			theStats::AddUpOverheadServer(packet->GetPacketSize());
 			SendPacket(packet, true);
 			#ifdef DEBUG_CLIENT_PROTOCOL
 			AddLogLineM(true,wxT("Client: OP_GETSERVERLIST"));
 			#endif
 		}
 	}
+	
+	theApp->ShowConnectionState();
 }
 
 
@@ -278,7 +293,8 @@ bool CServerConnect::SendPacket(CPacket* packet,bool delpacket, CServerSocket* t
 {
 	if (!to) {
 		if (connected) {
-			return connectedsocket->SendPacket(packet, delpacket, true);
+			connectedsocket->SendPacket(packet, delpacket, true);
+			return true;
 		} else {
 			if ( delpacket ) {
 				delete packet;
@@ -287,46 +303,46 @@ bool CServerConnect::SendPacket(CPacket* packet,bool delpacket, CServerSocket* t
 			return false;
 		}
 	} else {
-		return to->SendPacket(packet, delpacket, true);
+		to->SendPacket(packet, delpacket, true);
+		return true;
 	}
 }
 
 
-bool CServerConnect::SendUDPPacket(CPacket* packet, CServer* host, bool delpacket)
+bool CServerConnect::SendUDPPacket(CPacket* packet, CServer* host, bool delpacket, bool rawpacket, uint16 port_offset)
 {
 	if (connected) {
-		serverudpsocket->SendPacket(packet, host);
-	}
-
-	if (delpacket)
+		serverudpsocket->SendPacket(packet, host, delpacket, rawpacket, port_offset);
+	} else if (delpacket) {
 		delete packet;
+	}
 
 	return true;
 }
 
 
-void CServerConnect::ConnectionFailed(CServerSocket* sender){
+void CServerConnect::ConnectionFailed(CServerSocket* sender)
+{
 	if (connecting == false && sender != connectedsocket)
 	{
 		// just return, cleanup is done by the socket itself
 		return;
 	}
 	//messages
-	CServer* update = NULL;
+	CServer* pServer = theApp->serverlist->GetServerByAddress(sender->cur_server->GetAddress(), sender->cur_server->GetPort());
 	switch (sender->GetConnectionState()){
 		case CS_FATALERROR:
 			AddLogLineM(true, _("Fatal Error while trying to connect. Internet connection might be down"));
 			break;
 		case CS_DISCONNECTED:
-			theApp.sharedfiles->ClearED2KPublishInfo();
+			theApp->sharedfiles->ClearED2KPublishInfo();
 			AddLogLineM(false,CFormat( _("Lost connection to %s (%s:%i)") )
 				% sender->cur_server->GetListName()
 				% sender->cur_server->GetFullIP()
 				% sender->cur_server->GetPort() );
 
-			update = theApp.serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
-			if (update){
-				Notify_ServerHighlight(update, false);
+			if (pServer){
+				Notify_ServerHighlight(pServer, false);
 			}
 			break;
 		case CS_SERVERDEAD:
@@ -335,10 +351,9 @@ void CServerConnect::ConnectionFailed(CServerSocket* sender){
 				% sender->cur_server->GetFullIP()
 				% sender->cur_server->GetPort() );
 
-			update = theApp.serverlist->GetServerByAddress( sender->cur_server->GetAddress(), sender->cur_server->GetPort() );
-			if (update) {
-				update->AddFailedCount();
-				Notify_ServerRefresh( update );
+			if (pServer) {
+				pServer->AddFailedCount();
+				Notify_ServerRefresh( pServer );
 			}
 			break;
 		case CS_ERROR:
@@ -363,30 +378,27 @@ void CServerConnect::ConnectionFailed(CServerSocket* sender){
 			bool autoretry= !singleconnecting;
 			StopConnectionTry();
 			if ((thePrefs::Reconnect()) && (autoretry) && (!m_idRetryTimer.IsRunning())){ 
-				AddLogLineM(false, wxString::Format(_("Automatic connection to server will retry in %d seconds"), CS_RETRYCONNECTTIME)); 
-				
-				m_idRetryTimer.SetOwner(&theApp,TM_TCPSOCKET);
+				AddLogLineM(false, wxString::Format(wxPLURAL("Automatic connection to server will retry in %d second", "Automatic connection to server will retry in %d seconds", CS_RETRYCONNECTTIME), CS_RETRYCONNECTTIME)); 
 				m_idRetryTimer.Start(1000*CS_RETRYCONNECTTIME);
 			}
-			Notify_ShowConnState(false,wxEmptyString);
 			break;
 		}
 		case CS_DISCONNECTED:{
-			theApp.sharedfiles->ClearED2KPublishInfo();		
+			theApp->sharedfiles->ClearED2KPublishInfo();		
 			connected = false;
 			Notify_ServerHighlight(sender->cur_server,false);
-			if (connectedsocket) 
+			if (connectedsocket)  {
 				connectedsocket->Close();
+			}
 			connectedsocket = NULL;
-			theApp.searchlist->StopGlobalSearch();			
+			theApp->searchlist->StopGlobalSearch();			
 			Notify_SearchCancel();
-			theApp.statistics->SetServerConnectTime(0);
+			theStats::GetServerConnectTimer()->StopTimer();
 			if (thePrefs::Reconnect() && !connecting){
 				ConnectToAnyServer();		
 			}
 			
 			AddLogLineM( true, _("Connection lost") );
-			Notify_ShowConnState(false,wxEmptyString);
 			break;
 		}
 		case CS_ERROR:
@@ -400,14 +412,21 @@ void CServerConnect::ConnectionFailed(CServerSocket* sender){
 		}
 		case CS_SERVERDEAD:
 		case CS_SERVERFULL:{
-			if (!connecting)
+			if (!connecting) {
 				break;
-			if (singleconnecting){
+			}
+			if (singleconnecting) {
+				if (pServer && sender->IsServerCryptEnabledConnection() && !thePrefs::IsClientCryptLayerRequired()){
+					// try reconnecting without obfuscation
+					ConnectToServer(pServer, false, true);
+					break;
+				}
+				
 				StopConnectionTry();
 				break;
 			}
 
-			std::map<uint32, CServerSocket*>::iterator it = connectionattemps.begin();
+			ServerSocketMap::iterator it = connectionattemps.begin();
 			while ( it != connectionattemps.end() ){
 				if ( it->second == sender ) {
 					connectionattemps.erase( it );
@@ -418,17 +437,17 @@ void CServerConnect::ConnectionFailed(CServerSocket* sender){
 			TryAnotherConnectionrequest();
 		}
 	}
-	Notify_ShowConnState(false,wxEmptyString);
+	theApp->ShowConnectionState();
 }
 
 void CServerConnect::CheckForTimeout()
 {
 	uint32 dwCurTick = GetTickCount();
 
-	std::map<uint32, CServerSocket*>::iterator it = connectionattemps.begin();
+	ServerSocketMap::iterator it = connectionattemps.begin();
 	while ( it != connectionattemps.end() ){
 		if ( !it->second ) {
-			AddLogLineM(false, _("Error: Socket invalid at timeoutcheck"));
+			AddLogLineM(false, _("ERROR: Socket invalid at timeout check"));
 			connectionattemps.erase( it );
 			return;
 		}
@@ -458,27 +477,28 @@ void CServerConnect::CheckForTimeout()
 bool CServerConnect::Disconnect()
 {
 	if (connected && connectedsocket) {
-		theApp.sharedfiles->ClearED2KPublishInfo();
+		theApp->sharedfiles->ClearED2KPublishInfo();
 
 		connected = false;
 
-		CServer* update = theApp.serverlist->GetServerByAddress(
+		CServer* update = theApp->serverlist->GetServerByAddress(
 			connectedsocket->cur_server->GetAddress(),
 			connectedsocket->cur_server->GetPort());
 		Notify_ServerHighlight(update, false);
-		theApp.SetPublicIP(0);
+		theApp->SetPublicIP(0);
 		DestroySocket(connectedsocket);
 		connectedsocket = NULL;
-		Notify_ShowConnState(false,wxEmptyString);
-		theApp.statistics->SetServerConnectTime(0);
+		theApp->ShowConnectionState();
+		theStats::GetServerConnectTimer()->StopTimer();
 		return true;
-	}
-	else
+	} else {
 		return false;
+	}
 }
 
 
 CServerConnect::CServerConnect(CServerList* in_serverlist, amuleIPV4Address &address)
+: m_idRetryTimer(theApp,ID_SERVER_RETRY_TIMER_EVENT)
 {
 	connectedsocket = NULL;
 	used_list = in_serverlist;
@@ -489,11 +509,7 @@ CServerConnect::CServerConnect(CServerList* in_serverlist, amuleIPV4Address &add
 	singleconnecting = false;
 
 	// initalize socket for udp packets
-//#ifdef TESTING_PROXY
-	serverudpsocket = new CServerUDPSocket(this, address, thePrefs::GetProxyData());
-	m_idRetryTimer.SetOwner(&theApp,TM_TCPSOCKET);
-	lastStartAt=0;	
-	InitLocalIP();	
+	serverudpsocket = new CServerUDPSocket(address, thePrefs::GetProxyData());
 }
 
 
@@ -506,7 +522,6 @@ CServerConnect::~CServerConnect()
 	DestroySocket(connectedsocket);
 	connectedsocket = NULL;
 	// close udp socket
-	serverudpsocket->Close();
 	delete serverudpsocket;
 }
 
@@ -524,23 +539,18 @@ void CServerConnect::SetClientID(uint32 newid)
 {
 	clientid = newid;
 	
-	if (!IsLowIDED2K(newid)) {
-		theApp.SetPublicIP(newid);
+	if (!::IsLowID(newid)) {
+		theApp->SetPublicIP(newid);
 	}
-
-	Notify_ShowConnState(IsConnected(),GetCurrentServer()->GetListName());
 }
 
 
-void CServerConnect::DestroySocket(CServerSocket* pSck){
-	if (pSck == NULL)
+void CServerConnect::DestroySocket(CServerSocket* pSck)
+{
+	if (pSck == NULL) {
 		return;
-	// remove socket from list of opened sockets
-	POSITION pos = m_lstOpenSockets.Find( pSck );
-	if ( pos != NULL ) {
-		m_lstOpenSockets.RemoveAt( pos );
 	}
-
+	m_lstOpenSockets.remove(pSck);
 	pSck->Destroy();
 }
 
@@ -564,11 +574,10 @@ void CServerConnect::KeepConnectionAlive()
 		// "Ping" the server if the TCP connection was not used for the specified interval with
 		// an empty publish files packet -> recommended by lugdunummaster himself!
 		
-		CSafeMemFile* files = new CSafeMemFile(4);
-		files->WriteUInt32(0); //nFiles
+		CMemFile files(4);
+		files.WriteUInt32(0); //nFiles
 	
-		CPacket* packet = new CPacket(files);
-		packet->SetOpCode(OP_OFFERFILES);
+		CPacket* packet = new CPacket(files, OP_EDONKEYPROT, OP_OFFERFILES);
 		#ifdef DEBUG_CLIENT_PROTOCOL
 		AddLogLineM(true,wxT("Client: OP_OFFERFILES"));
 		#endif
@@ -578,24 +587,48 @@ void CServerConnect::KeepConnectionAlive()
 		//   - this function is called once when connecting to a server and when a file becomes shareable - so, it's called rarely.
 		//   - if the compressed size is still >= the original size, we send the uncompressed packet
 		// therefor we always try to compress the packet
-		theApp.statistics->AddUpDataOverheadServer(packet->GetPacketSize());
+		theStats::AddUpOverheadServer(packet->GetPacketSize());
 		connectedsocket->SendPacket(packet,true);
 		
 		AddDebugLogLineM(false, logServer, wxT("Refreshing server connection"));
-		delete files;
  	}
 }
 
-
-void CServerConnect::InitLocalIP()
+// true if the IP is one of a server which we currently try to connect to
+bool CServerConnect::AwaitingTestFromIP(uint32 dwIP)
 {
-	m_nLocalIP = 0;
+	ServerSocketMap::iterator it = connectionattemps.begin();
 
-	try{
-		m_nLocalIP = StringHosttoUint32(GetLocalHost());
+	while (it != connectionattemps.end()) {
+		const CServerSocket* serversocket = it->second;
+		if (serversocket && (serversocket->GetConnectionState() == CS_WAITFORLOGIN) &&
+			(serversocket->GetServerIP() == dwIP)) {
+			return true;
+		}
+		++it;
 	}
-	catch(...){
-		// at least two ppl reported crashs when using 'gethostbyname' with third party winsock DLLs
-		AddDebugLogLineM(false, logGeneral, wxT("Unknown exception in CServerConnect::InitLocalIP"));
+	
+	return false;
+}
+
+
+bool CServerConnect::IsConnectedObfuscated() const
+{
+	return connectedsocket != NULL && connectedsocket->IsObfusicating();
+}
+
+
+void CServerConnect::OnServerHostnameResolved(void* socket, uint32 ip)
+{
+	// The socket object might have been deleted while we waited for the hostname
+	// to be resolved, so we check if it's still among the open sockets.
+	SocketsList::iterator it = std::find(m_lstOpenSockets.begin(), m_lstOpenSockets.end(), socket);
+	if (it != m_lstOpenSockets.end()) {
+		(*it)->OnHostnameResolved(ip);
+	} else {
+		printf("Received late result of DNS lookup, discarding.\n");
 	}
 }
+
+
+// File_checked_for_headers
