@@ -1,7 +1,7 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -20,18 +20,12 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 #ifndef SERVERWND_H
 #define SERVERWND_H
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma interface "ServerWnd.h"
-#endif
-
-#include <wx/defs.h>		// Needed before any other wx/*.h
-#include <wx/panel.h>		// Needed for wxPanel
 #include <wx/splitter.h>	// Needed for wxSplitter
 
 
@@ -44,13 +38,15 @@ public:
 	virtual ~CServerWnd();
 
 	void UpdateServerMetFromURL(const wxString& strURL);
-	void UpdateMyInfo();
+	void UpdateED2KInfo();
+	void UpdateKadInfo();
 
 	CServerListCtrl* serverlistctrl;
 
 private:
 	void OnSashPositionChanged(wxSplitterEvent& evt);
 	void OnBnClickedAddserver(wxCommandEvent& evt);
+	void OnBnClickedED2KDisconnect(wxCommandEvent& evt);
 	void OnBnClickedUpdateservermetfromurl(wxCommandEvent& evt);
 	void OnBnClickedResetLog(wxCommandEvent& evt);
 	void OnBnClickedResetServerLog(wxCommandEvent& evt);
@@ -59,3 +55,4 @@ private:
 };
 
 #endif // SERVERWND_H
+// File_checked_for_headers
