@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2006 Kry ( elkry@users.sourceforge.net / http://www.amule.org )
-// Copyright (c) 2003-2006 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2008 Kry ( elkry@users.sourceforge.net / http://www.amule.org )
+// Copyright (c) 2003-2008 aMule Team ( admin@amule.org / http://www.amule.org )
 // Copyright (c) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
@@ -27,42 +27,18 @@
 #ifndef WEBSERVER_H
 #define WEBSERVER_H
 
-#ifdef __WXMSW__
-	#include <wx/msw/winundef.h>
-#endif
-
-#include <wx/wx.h>
-
-#ifdef __WXMSW__
-	#include <winsock.h>
-#else
-	#include <sys/types.h>
-	#include <sys/socket.h>
-	#include <netinet/in.h>
-	#include <arpa/inet.h>
-#endif
-#include <zlib.h>		// Needed for Bytef etc.
-
 #ifdef WITH_LIBPNG
 	#include <png.h>
 #endif
 
-#include <list>
-#include <map>
-#include <vector>
-
-#include <wx/filename.h>
-
-#include <ec/ECPacket.h>
-
-#include "Types.h"
-#include "OtherFunctions.h"
 #include "WebInterface.h"
 #include "KnownFile.h"
 #include "RLE.h"
 #include "OtherStructs.h"
 
-#include "kademlia/utils/UInt128.h" // Need for UInt128
+#include "UPnP.h"
+
+#include <wx/datetime.h>  // For DownloadFile::wxtLastSeenComplete
 
 //class TransferredData;
 class CWSThread;
@@ -90,9 +66,9 @@ class DownloadFile {
 	public:
 		wxString	sFileName;
 		uint8		nFileStatus;
-		unsigned long	lFileSize;
-		unsigned long	lFileCompleted;
-		unsigned long	lFileTransferred;
+		uint64		lFileSize;
+		uint64		lFileCompleted;
+		uint64		lFileTransferred;
 		unsigned long	lFileSpeed;
 		long		lSourceCount;
 		long		lNotCurrentSourceCount;
@@ -104,6 +80,7 @@ class DownloadFile {
 		wxString	sFileHash;
 		wxString	sED2kLink;
 		uint8		nCat;
+		wxDateTime	wxtLastSeenComplete;
 
 		CMD4Hash	nHash;
 		
@@ -121,8 +98,8 @@ class DownloadFile {
 class SharedFile {
 	public:
 		wxString	sFileName;
-		unsigned long	lFileSize;
-		uint32		nFileTransferred;
+		uint64		lFileSize;
+		uint64		nFileTransferred;
 		uint64		nFileAllTimeTransferred;
 		uint16		nFileRequests;
 		uint32		nFileAllTimeRequests;
@@ -699,13 +676,23 @@ struct ThreadData {
 	CWebSocket	*pSocket;
 };
 
-/*
- * In transition period I want both versions of amuleweb available: CPP and PHP
- */
-class CWebServerBase {
+enum {
+    // Socket handlers
+    ID_WEBLISTENSOCKET_EVENT = wxID_HIGHEST+123,  // random safe ID
+    ID_WEBCLIENTSOCKET_ENENT,
+};
+
+#ifdef ENABLE_UPNP
+class CUPnPControlPoint;
+class CUPnPPortMapping;
+#endif
+
+class CWebServerBase : public wxEvtHandler {
 	protected:
-		CWSThread *wsThread;
-		wxMutex *m_mutexChildren;
+		//CWSThread *wsThread;
+		wxSocketServer *m_webserver_socket;
+		
+		wxMutex m_mutexChildren;
 
 		ServersInfo m_ServersInfo;
 		SharedFileInfo m_SharedFileInfo;
@@ -718,23 +705,32 @@ class CWebServerBase {
 		CImageLib m_ImageLib;
 
 		virtual void ProcessURL(ThreadData) = 0;
-		void ProcessImgFileReq(ThreadData);
+		virtual void ProcessImgFileReq(ThreadData) = 0;
 
 		int GzipCompress(Bytef *dest, uLongf *destLen,
 			const Bytef *source, uLong sourceLen, int level);
 			
-	friend class CWebSocket;
-	friend class CWSThread;		// to access the wsThread member
-	friend class CPhPLibContext;
+		friend class CWebSocket;
+		friend class CPhPLibContext;
 	
+		bool m_upnpEnabled;
+		int m_upnpTCPPort;
+#ifdef ENABLE_UPNP
+		CUPnPControlPoint *m_upnp;
+		std::vector<CUPnPPortMapping> m_upnpMappings;
+#endif
+
+		void OnWebSocketServerEvent(wxSocketEvent& event);
+		void OnWebSocketEvent(wxSocketEvent& event);
+		DECLARE_EVENT_TABLE();
 	public:
 		CWebServerBase(CamulewebApp *webApp, const wxString& templateDir);
 		virtual ~CWebServerBase() { }
 
 		void Send_Discard_V2_Request(CECPacket *request);
 
-		virtual void StartServer() = 0;
-		virtual void StopServer() = 0;
+		void StartServer();
+		void StopServer();
 
 		void Print(const wxString &s);
 
@@ -794,12 +790,11 @@ class CScriptWebServer : public CWebServerBase {
 		CSession *CheckLoggedin(ThreadData &);
 	protected:
 		virtual void ProcessURL(ThreadData);
+		virtual void ProcessImgFileReq(ThreadData);
 	public:
 		CScriptWebServer(CamulewebApp *webApp, const wxString& templateDir);
 		~CScriptWebServer();
 
-		virtual void StartServer();
-		virtual void StopServer();
 };
 
 class CNoTemplateWebServer : public CScriptWebServer {
@@ -811,3 +806,4 @@ class CNoTemplateWebServer : public CScriptWebServer {
 };
 
 #endif // WEBSERVER_H
+// File_checked_for_headers
